<?php

namespace BulkUpload\Handlers;

use BulkUpload\Interfaces\FileInterface;
use BulkUpload\Error\Error;
use Shuchkin\SimpleCSV;
use Shuchkin\SimpleXLS;
use Shuchkin\SimpleXLSX;

class File implements FileInterface{
    
    use \BulkUpload\Traits\FileProcessor;

    private \collection $db;

    public function __construct()
    {
        $this->db = new \collection;
    }

    /**
     * Retrieves a file located on the $fileFolder, with the given $fileIdentifier and the extension as $extension. Returns the file complete address.
     *
     * @param string $fileFolder
     * @param string $fileIdentifier
     * @param string $extension
     * @return string|bool|Error
     */
    public function getFile(string $bulkToken, string $filePurpose, string $extension, string $name):string|bool|Error{

        $whereData = [
            'bulk_upload_id' => $bulkToken,
            'file_purpose' => $filePurpose
        ];

        $results = $this->db->search('ng_bulk_upload_files', $whereData, limit:1);

        if($results['cant_rows'] < 1){
            return new Error('GetFile', 'There is no file with the given token and purpose on the database.');
        }

        $data = json_decode($results[0]['file_data'], 'ISO-8859-1');

        $file = match($extension){
            'csv' => $this->fileToCSV($data),
            'xls' => $this->filetoXLSX($data, $name),
            'xlsx' => $this->filetoXLSX($data, $name)
        };

        return $file;

    }

    /**
     * Saves a file in the given $fileFolder, named as $fileIdentifier and with the $extension extension. Returns true if the save process was successful.
     *
     * @param string $fileFolder
     * @param string $fileIdentifier
     * @param string $extension
     * @return boolean
     */
    public function saveFile(string $bulkToken, string $filePurpose, array $fileData, string $fileType, string $csv_separator = "", int $is_updatable = 0):bool
    {

        $extension = $this->getFileExtension($fileData['name']);

        if($extension == 'csv' && $csv_separator == ""){
            throw new \Exception("Si el archivo es CSV, debe de proveer un separador de columnas.", 1);
        }
        $data = match($extension){
            'xlsx' => SimpleXLSX::parse($fileData['tmp_name'])->rows(),
            'xls' => SimpleXLS::parse($fileData['tmp_name'])->rows(),
            'csv' => SimpleCSV::import($fileData['tmp_name'], delimiter:$csv_separator),
        };

        $requestData = [
            'bulk_upload_id' => $bulkToken,
            'file_purpose' => $filePurpose,
            'file_data' => json_encode($data, JSON_UNESCAPED_UNICODE),
            'is_updatable' => $is_updatable,
            'file_type' => $fileType,
            'csv_separator' => $csv_separator,
        ];

        if($this->db->insert('ng_bulk_upload_files', $requestData)){
            return true;
        }
        
        return false;

    }

    public function saveBulkUploadFile(array $file, string $purpose = "unprocessed", string $csv_separator = "", int $is_updatable = 0):array
    {

        $extension = $this->validateFile($file, 'bulk-upload', $csv_separator);

        if($extension == 'csv' && $csv_separator == ""){
            throw new \Exception("Si el archivo es CSV, debe de proveer un separador de columnas.", 1);
        }

        $hash = wp_generate_uuid4();
        if($this->saveFile($hash, $purpose, $file, $extension, $csv_separator, $is_updatable)){
            return ["hash" => $hash, "status" => true, "columns" => $this->getFileHeaders($file), "is_updatable" => $is_updatable];
        } 
        return ["hash" => "", "status" => false, "columns" => [], "is_updatable" => $is_updatable];

    }

    public function updateColumnMap(array $column_map, string $hash):array
    {
        if(!$this->validateColumnMap($column_map)){
            throw new \Exception("El mapeo de columnas es inválido, hay asignaciones duplicadas o vacías.", 1);
            return ["hash" => "", "status" => false, "column_map" => json_encode($column_map, JSON_UNESCAPED_UNICODE)];
        }
        $update = [
            "key_map" => json_encode($column_map, JSON_UNESCAPED_UNICODE)
        ];
        $where = [
            'bulk_upload_id' => $hash,
        ];
        if($this->db->update('ng_bulk_upload_files', $where, $update)){
            return ["hash" => $hash, "status" => true, "column_map" => json_encode($column_map, JSON_UNESCAPED_UNICODE)];
        }
        return ["hash" => "", "status" => false, "column_map" => json_encode($column_map, JSON_UNESCAPED_UNICODE)];
        
    }

    private function validateColumnMap(array $column_map):bool
    {
        $count = array_count_values($column_map);
        
        //die(var_dump($count));
        foreach ($count as $counted) {
            if($counted <> 1 ){
                return false;
            }
        }
        return true;
        
    }

    private function getFileHeaders(array $file):array
    {

        $data = match($this->getFileExtension($file['name'])){
            'xlsx' => SimpleXLSX::parse($file['tmp_name'])->rows(),
            'xls' => SimpleXLS::parse($file['tmp_name'])->rows(),
            'csv' => SimpleCSV::import($file['tmp_name']),
        };
        return $data[0];

    }

    private function validateFile(array $file, string $type = "basic", string $csv_separator = ""):string
    {
        switch ($type) {
            case 'basic':
                $extension = $this->basicValidation($file);
                return $extension;
            case 'bulk-upload':
                $extension = $this->bulkUploadValidation($file, $csv_separator);
                return $extension;
            default:
                throw new \Exception("No ha sido indicado el tipo de validación de archivo.", 1);
                
        }

    }

    private function bulkUploadValidation(array $file, string $csv_separator = ""):string
    {
        $extension = $this->extensionValidation($file);
        $this->sizeValidation($file, 10000000);
        $this->lineValidator($file, 2, 101, $csv_separator);
        //$this->uniqueSkuValidator($file, );
        return $extension;
    }

    private function basicValidation(array $file):string
    {

        return "";
    }

    private function extensionValidation(array $file):string
    {
        $extension = $this->getFileExtension($file['name']);
        if(!in_array($extension, ['xls', 'xlsx', 'csv'])){
            throw new \Exception("La extensión del archivo no es válida, solo son válidas las extensiones de tipo: .csv, .xls y .xlsx.", 1);
        }
        return $extension;
    }

    private function sizeValidation(array $file, int $size):void
    {
        if($file['size'] > $size){
            throw new \Exception("El tamaño del archivo excede el límite de ".$size." bytes.", 1);
        }
        return;
    }

    private function lineValidator(array $file, int $min = 1, int $max = 100, string $csv_separator = ""):void
    {
        
        $extension = $this->getFileExtension($file['name']);

        if($extension == 'csv' && $csv_separator == ""){
            throw new \Exception("Si el archivo es CSV, debe de proveer un separador de columnas.", 1);
        }
        $data = match($extension){
            'xlsx' => SimpleXLSX::parse($file['tmp_name'])->rows(),
            'xls' => SimpleXLS::parse($file['tmp_name'])->rows(),
            'csv' => SimpleCSV::import($file['tmp_name'], delimiter:$csv_separator),
        };

        if(count($data) < $min){
            throw new \Exception("La cantidad de líneas de producto del archivo debería ser de al menos ".$min, 1);
        }

        if(count($data) > $max){
            throw new \Exception("La cantida de líneas de producto del archivo debería ser de un máximo de ".$max, 1);
        }
        return;

    }

    private function uniqueSkuValidator(array $file, string $csv_separator = ""):void
    {

        $extension = $this->getFileExtension($file['name']);

        if($extension == 'csv' && $csv_separator == ""){
            throw new \Exception("Si el archivo es CSV, debe de proveer un separador de columnas.", 1);
        }

        $data = match($extension){
            'xlsx' => SimpleXLSX::parse($file['tmp_name'])->rows(),
            'xls' => SimpleXLS::parse($file['tmp_name'])->rows(),
            'csv' => SimpleCSV::import($file['tmp_name'], delimiter:$csv_separator),
        };

        // $skuCounter = array_map($data)

        // if(count($data) - 1 != $skuCounter){
        //     throw new \Exception("No pueden existir SKU's repetidos dentro del mismo archivo.", 1);
            
        // }

    }




    public function setHeaders(string $name, string $type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', string $token = "template", string $purpose = "template", string $fileType ='xlsx'){   
        ob_start();
        $doc = $this->getFile($token, $purpose, $fileType, $name.".".$fileType);
        ob_get_clean();
        header('Access-Control-Allow-Origin: *');
        header('Access-Control-Allow-Methods: GET');
        header("Access-Control-Allow-Headers: X-Requested-With");
        header('Content-Type: '.$type);
        header('Cache-Control: public, must-revalidate, max-age=0');
        header('Pragma: public');
        header('Expires: Sat, 26 Jul 1997 05:00:00 GMT');
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
        header("Content-Disposition:attachment;filename=".$name.".".$fileType);
        readfile($doc);
        die();
    }

}