<?php

namespace BulkUpload\Terms;

use BulkUpload\Error\Error;
use BulkUpload\Handlers\MetaData;
use BulkUpload\Interfaces\TermInterface;
use \collection;

class Brand implements TermInterface{

    public int $term_id;
    public int $term_taxonomy_id;
    public string $name;
    public string $slug;
    public string $taxonomy = "product_brand";
    public ?int $count; 
    
    /**
     * Instance of the database manager
     *
     * @var \collection
     */
    private collection $db;

    public function __construct(array $term_data)
    {
        $this->term_id = $term_data['term_id'];
        $this->term_taxonomy_id = $term_data['term_taxonomy_id'];
        $this->name = $term_data['name'];
        $this->slug = $term_data['slug'];
        $this->count = $term_data['count'];
        $this->db = new collection;
    }

    /**
     * Returns the term id.
     *
     * @return integer
     */
    public function getTermTermId()
    {
        return $this->term_id;
    }

    /**
     * Returns the term name.
     *
     * @return string
     */
    public function getTermName()
    {
        return $this->name;
    }

    /**
     * Returns a Category|Tag class that matches with the provided id.
     *
     * @param integer $term_id
     * @return Brand|Error
     */
    static public function getTermFromId(int $term_id){

        $db =  new collection;

        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_brand'
                    AND OvMdW_terms.`term_id` = '".$db->sanitizeData($term_id)."';
        ");
        $result = $db->plainQuery($query);

        if($result['cant_rows'] < 1){
            return new Error("Brand", "There is no brand with the provided term_id"); 
        }

        return new Brand($result[0]);

    }
    
    /**
     * Returns a Category|Tag class that matches with the provided name and type.
     *
     * @param string $name
     * @param string $type
     * @return Brand|Error
     */
    static public function getTermFromName(string $name){

        $db =  new collection;
        $name = strtoupper($name);

        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_brand'
                    AND UPPER(OvMdW_terms.`name)` = '".$db->sanitizeData($name)."';
        ");
        $result = $db->plainQuery($query);

        if($result['cant_rows'] < 1){
            return new Error("Brand", "There is no brand with the provided name"); 
        }

        return new Brand($result[0]);
        
    }

    static public function getTermFromSlug(string $slug):Brand|Error{

        $db = new collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_brand'
                    AND OvMdW_terms.`slug` = '".$db->sanitizeData($slug)."';
        ");

        $result = $db->plainQuery($query);
        if($result['cant_rows'] < 1){
            return new Error("Category", "There is no category with the provided slug");
        }

        return new Brand($result[0]);

    }

    /**
     * Retrieves true if the provided brand exists and false if not. It's case sensitive.
     *
     * @param string $brand_name
     * @return boolean
     */
    static public function checkIfBrandExists(string $brand_name):bool{

        $db =  new collection;

        $query = str_replace('OvMdW_', TABLE_PREFIX, 'SELECT 
            `OvMdW_terms`.`name`,
            `OvMdW_terms`.`term_id`, 
            `OvMdW_term_taxonomy`.`taxonomy` 
        FROM 
            `OvMdW_terms` 
        INNER JOIN 
            `OvMdW_term_taxonomy` ON `OvMdW_terms`.`term_id` = `OvMdW_term_taxonomy`.`term_id` 
        WHERE 
            `taxonomy` = "product_brand" 
            AND `OvMdW_terms`.`name` = "'.$db->sanitizeData($brand_name).'" 
        LIMIT 1');
        $result =$db->plainQuery($query);

        if($result['cant_rows'] < 1){
            return false;
        }

        return true;

    }

    /**
     * Generates a new brand request based on the $brand_name and assigns that request to the given $product_id.
     *
     * @param string $brand_name
     * @param integer $product_id
     * @return void
     */
    static public function requestNewBrand(string $brand_name, int $product_id){

        MetaData::insertMetaData($product_id, '_custom_product_brand', $brand_name);

    }

    static public function brandExists(string $brand_name):bool
    {

        $db = new collection;
        $brand_name = strtoupper($brand_name);

        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_brand'
                    AND UPPER(OvMdW_terms.`name)` = '".$db->sanitizeData($brand_name)."';
        ");
        $result = $db->plainQuery($query);

        if($result['cant_rows'] < 1){
            return false;
        }

        return true;

    }

}